// Copyright 1997
// College of Computer Science
// Northeastern University Boston MA 02115

// This software may be used for educational purposes as long as this copyright
// notice is retained at the top of all files

// Should this software be modified, the words "Modified from Original" must be
// included as a comment below this notice

// All publication rights are retained.  This software or its documentation may
// not be published in any media either in whole or in part.

///////////////////////////////////////////////////////////////////////////////

// Delay.h

///////////////////////////////////////////////////////////////////////////////

#ifndef DELAY_H_
#define DELAY_H_

// #include <bool.h>		Omit!

#include <string>			// C++ string class


// define tick = 1/60 second

extern long DelayTicks;		// default delay ticks

extern bool ManualMode;		// default pause setting

extern bool OnButtonFreeze;	// default action on mouse press

// flags for actual key states

extern bool CommandFlag;	// is command down?
extern bool OptionFlag;		// is option down?
extern bool ControlFlag;	// is control down?
extern bool ShiftFlag;		// is shift down?
extern bool CapsLockFlag;	// is caps lock down?

// flags to signal action

extern bool BreakFlag;		// convention: true if command down
extern bool SpeedFlag;		// convention: true if caps lock down
extern bool AbortFlag;		// convention: true if command-option-control down


// class to throw on user decision to abort
// abort signaled by pressing command-option-control

class UserAbort { };

// UserAbort is thrown directly by ResetFlags
// ResetFlags is called by Delay, Pause, & Await routines


// Delay and pause philosophy common to Delay, Pause, & Await routines
//
// Exit immediately if on entry:
//		caps lock down and mouse up
//
// Mouse pause interaction:
//		if mouse button is down or goes down
//		while in delay or pause cycle
//		pause until mouse buttom is released
//		then exit immediately afterwards


// delay routines

void Delay(long ticks = DelayTicks);
// delay by tick count specified


void SetDelay(long ticks);
// set default delay ticks


long GetDelay();
// get default delay ticks


void RequestDelay(const string& prompt = "Delay", long answer = DelayTicks);
// request default delay ticks from user


// pause routines

void Pause();
// if Manual Mode is true
//		call AwaitClick
// else
//		call Delay


void SetPause(bool mode);
// set Manual Mode state


bool GetPause();
// get Manual Mode state


void RequestPause(const string& prompt = "Pause", bool answer = ManualMode);
// request Manual Mode setting from user
// if Manual Mode is set to false then
// request default delay ticks from user


// mouse utilities
// note: to avoid switching applications, click mouse in drawing window!
// note: for related utilities see: Mouse.h

void AwaitPress();
// wait for mouse down


void AwaitRelease();
// if mouse down then wait for mouse up


void AwaitClick();
// wait for mouse down and for mouse up


// key press utilities
// abort signaled by pressing command-option-control

short ModifierStatus();
// return status of five keyboard modifier flags in short
// bit  8:	key: command	maskname: cmdKey
// bit  9:	key: shift		maskname: shiftKey
// bit 10:	key: caps lock	maskname: alphaLock
// bit 11:	key: option		maskname: optionKey
// bit 12:	key: control	maskname: controlKey
//
// ModifierStatus is similar to modifiers field of an EventRecord
// except that bits 0 and 7 are not set to indicate activate/mouse status


void ResetFlags();
// use ModifierStatus to set current state of flags

#endif // DELAY_H_