// Copyright 1997
// College of Computer Science
// Northeastern University Boston MA 02115

// This software may be used for educational purposes as long as this copyright
// notice is retained at the top of all files

// Should this software be modified, the words "Modified from Original" must be
// included as a comment below this notice

// All publication rights are retained.  This software or its documentation may
// not be published in any media either in whole or in part.

///////////////////////////////////////////////////////////////////////////////

// FileTmpl.h
// Templates for binary io

///////////////////////////////////////////////////////////////////////////////

// Changes: Summer 1997
// Update file includes
// Omit byte definition

///////////////////////////////////////////////////////////////////////////////

#ifndef FILETMPL_H_
#define FILETMPL_H_

#include <iomanip.h>					// io manipulators
#include <fstream.h>					// file stream
#include <sstream.h>					// string stream

#include <string>						// C++ string class definition


// Omit: Summer 1997
// Define byte as an abbreviation for unsigned char
// typedef unsigned char byte;


// Help routines
// Create special functions to handle long parameters in case int and long are distinct
// This is needed since C++ standard library passes count as an int rather than a long!
// These help routines are not intended to be called externally!


// ReadArrayHelp(In, charpointer, count)
inline void ReadArrayHelp(istream& In, char* pointer, long count) {
	if (sizeof(int) == sizeof(long))
		In.read(pointer, count);
	else {
		long block = 16384;
		long shift = 0;
		
		while (count > block) {
			In.read(pointer + shift, block);
			count -= block;
			shift += block;
		}
		
		In.read(pointer + shift, count);
	}
}


// WriteArrayHelp(Out, charpointer, count)
inline void WriteArrayHelp(ostream& Out, const char* pointer, long count) {
	if (sizeof(int) == sizeof(long))
		Out.write(pointer, count);
	else {
		long block = 16384;
		long shift = 0;
		
		while (count > block) {
			Out.write(pointer + shift, block);
			count -= block;
			shift += block;
		}
		
		Out.write(pointer + shift, count);
	}
}


// The main routines Read and Write for single objects


// Read(In, object)
// Read one object of type T from the binary input file stream In
template <class T> inline void Read(istream& In, T& object) {
	ReadArrayHelp(In, (char *)(&object), sizeof(T));
}


// Write(Out, object)
// Write one object of type T to the binary output file stream Out
template <class T> inline void Write(ostream& Out, const T& object) {
	WriteArrayHelp(Out, (const char *)(&object), sizeof(T));
}


// The routines ReadArray and WriteArray for arrays of objects


// ReadArray(In, pointer, count)
// Read an array of type T addressed by pointer and with count elements
// from the binary input file stream In
template <class T> inline void ReadArray (istream& In, T* pointer, long count) {
	ReadArrayHelp(In, (char *)(pointer), count * sizeof(T));
}


// WriteArray(Out, pointer, count)
// Write an array of type T addressed by pointer and with count elements
// to the binary output file stream Out
template <class T> inline void WriteArray(ostream& Out, const T* pointer, long count) {
	WriteArrayHelp(Out, (const char *)(pointer), count * sizeof(T));
}

#endif // FILETMPL_H_