// Copyright 1997
// College of Computer Science
// Northeastern University Boston MA 02115

// This software may be used for educational purposes as long as this copyright
// notice is retained at the top of all files

// Should this software be modified, the words "Modified from Original" must be
// included as a comment below this notice

// All publication rights are retained.  This software or its documentation may
// not be published in any media either in whole or in part.

///////////////////////////////////////////////////////////////////////////////

// FileTool.cp
// Macintosh Implementation

///////////////////////////////////////////////////////////////////////////////

// Changes: Summer 1997
// Use C++ string class instead of older CharBuffer template class

///////////////////////////////////////////////////////////////////////////////

#include "CHeaders.h"
#include "SWindows.h"
#include "FileTool.h"
#include "Strings.h"	// Macintosh C string to Pascal string converter


// hidden types

struct folderdata {
	char	name[64];	// folder name
	short	refnum;		// folder or volume reference number
	long	fid;		// folder id number
	long	pid;		// parent id number
};


// hidden prototypes

void GetFullName(FSSpec& info, string& FullName);

void GetFolderName(folderdata& data, string& FullName);

void GetFolderData(folderdata& data);


// implementation

void GetFullName(FSSpec& info, string& FullName) {
	char* name = (char*)(&info.name);
	p2cstr(StringPtr(name));
	
	folderdata data;
	data.refnum	= info.vRefNum;
	data.fid	= info.parID;
	
	GetFolderName(data, FullName);
	
	FullName += name;
};


void GetFolderName(folderdata& data, string& FullName) {
	GetFolderData(data);
	
	if (!data.name[0])
		FullName.erase();
		
	else {
		// add Maintosh directory separator (colon) to name
		int length = strlen(data.name);
		
		data.name[length]	= ':';
		data.name[length+1]	= '\0';
		
		// if at top level initialize FullName to name
		if (data.fid <= 2)
			FullName = data.name;
		
		// else do recursion to find higher level name
		// then append
		else {
			// recursion
			folderdata parent;
			parent.refnum	= data.refnum;
			parent.fid		= data.pid;
			GetFolderName(parent, FullName);
			
			// append
			FullName += data.name;
		}
	}
};


void GetFolderData(folderdata& data) {
	CInfoPBRec	info;
	
	data.name[0] = '\0';						// make it a null string both for C and Pascal
	
	// no io completion routine
	info.dirInfo.ioCompletion	=  0;
	
	// use directory ioDirID not initial ioNamePtr
	info.dirInfo.ioFDirIndex	= -1;
	
	// use name field in data as return area
	info.dirInfo.ioNamePtr		= StringPtr(data.name);
	
	// initial refnum
	info.dirInfo.ioVRefNum		=  data.refnum;
	
	// initial directory id
	info.dirInfo.ioDrDirID		= (data.fid < 0) ? 0 : data.fid;
	
	// Change: Summer 1997
	// Use PBGetCatInfoSync in place of obsolete call PBGetCatInfo
	OSErr error = PBGetCatInfoSync(&info);
	
	if (!error) {
		p2cstr(StringPtr(data.name));			// change name to c format
		data.fid = info.dirInfo.ioDrDirID;		// set directory id
		data.pid = info.dirInfo.ioDrParID;		// set parent directory id
		
		// cout << data.name << endl;			// debugging code
	}
}


bool SelectOldFileName(string& FullName) {
	StandardFileReply	reply;		// extended standard file reply struct
	SFTypeList			types;		// array of file types --- we do not use it
	
	SaveDrawWindow();				// save the drawing window
	
	// old file name dialog routine
	//  0 is a null FileFilterProcPtr
	// -1 indicates that types array will be ignored
	// reply is where we get the result of the dialog
	
	StandardGetFile(0, -1, types, &reply);
	
	RefreshDrawing();				// refresh the drawing window
	
	FullName.erase();				// initialize name
	
	if (reply.sfGood)				// user made a selection
		GetFullName(reply.sfFile, FullName);	// convert name to a full name
	
	return reply.sfGood;			// return 
}


bool SelectNewFileName(string& FullName, const string& FileName) {
	char prompt[1] = "";
	char filename[64];
	
	strcpy(filename, FileName.c_str());
	c2pstr(filename);
	
	StandardFileReply	reply;		// extended standard file reply struct
	
	SaveDrawWindow();				// save the drawing window
	
	// new file name dialog routine
	// reply is where we get the result of the dialog
	
	StandardPutFile(StringPtr(&prompt), StringPtr(&filename), &reply);
	
	RefreshDrawing();				// refresh the drawing window
	
	FullName.erase();				// initialize name
	
	if (reply.sfGood)				// user made a selection
		GetFullName(reply.sfFile, FullName);	// convert name to a full name
	
	return reply.sfGood;			// return 
};


bool OpenFile(ifstream& In, const char* Name,    const ios::openmode Mode) {
	// Attempt to open
	In.open(Name, Mode);
	
	// Check if opened successfully
	if (In.is_open()) {
		// Clear stale state bits from any prior open
		In.clear();
		
		return true;
	}
	else {
		// Write error message to cout
		cout << "Open Failure for File:" << endl;
		cout << Name << endl;
		cout << endl;
		
		return false;
	}
}


bool OpenFile(ifstream& In, const string& Name,  const ios::openmode Mode) {
	return OpenFile(In, Name.c_str(), Mode);
}


bool OpenFile(ofstream& Out, const char* Name,   const ios::openmode Mode) {
	// Attempt to open
	Out.open(Name, Mode);
	
	// Check if opened successfully
	if (Out.is_open()) {
		// Clear stale state bits from any prior open
		Out.clear();
		
		return true;
	}
	else {
		// Write error message to cout
		cout << "Open Failure for File:" << endl;
		cout << Name << endl;
		cout << endl;
		
		return false;
	}
}


bool OpenFile(ofstream& Out, const string& Name, const ios::openmode Mode) {
	return OpenFile(Out, Name.c_str(), Mode);
}


long FileSize(ifstream& In) {
	if (In.is_open()) {
		// access the underlying file buffer via a pointer
		filebuf* buffer = In.rdbuf();
		
		// save current position
		streampos current = buffer->pubseekoff(0, ios::cur);
		
		// seek to end of file
		streampos endfile = buffer->pubseekoff(0, ios::end);
		
		// seek current position
		buffer->pubseekpos(current);
		
		// return file size
		return endfile.offset();
	}
	else
		return 0;
}