// Copyright 1997
// College of Computer Science
// Northeastern University Boston MA 02115

// This software may be used for educational purposes as long as this copyright
// notice is retained at the top of all files

// Should this software be modified, the words "Modified from Original" must be
// included as a comment below this notice

// All publication rights are retained.  This software or its documentation may
// not be published in any media either in whole or in part.

///////////////////////////////////////////////////////////////////////////////

// Graphics.h

// This file contains basic graphics routines and RGB color routines
// This is the Apple Macintosh version
// Assume Macintosh Quickdraw definitions
// Must provide equivalent definitions if porting to another machine

///////////////////////////////////////////////////////////////////////////////

// Changes: Summer 1997
// Base RGB type on byte type defined as unsigned char
// Omit code to read RGB using names ... this code was never satisfactory!

///////////////////////////////////////////////////////////////////////////////

#ifndef GRAPHICS_H_
#define GRAPHICS_H_

// #include <bool.h>					Omit!
// #include "CHeaders.h"				Omit!

#include <string>						// C++ string class definition
#include "Mouse.h"						// For convenience graphics loads mouse


// define byte for use in RGB type below

typedef unsigned char byte;


// Basic graphics routines and definitions


// Black-white pattern type used in fill routines
// This is a Macintosh QuickDraw definition

typedef ConstPatternParam PatternPointer;


// Move and line draw routines
// We assume the existence of the four basic movement and line drawing routines:


// MoveTo(x, y);		// Move to absolute point (x, y)
// Move(dx, dy);		// Move by relative displacement (dx, dy)
// LineTo(x, y);		// Draw line to absolute point (x, y)
// Line(dx, dy);		// Draw line by relative displacement (dx, dy)


// Define correspoding routines which take Point parameters:

inline void MoveTo(const Point& P) {	// P = position
	MoveTo(P.h, P.v);
}


inline void Move(const Point& dP) {		// dP = change in position
	Move(dP.h, dP.v);
}


inline void LineTo(const Point& P) {	// P = position
	LineTo(P.h, P.v);
}


inline void Line(const Point& dP) {		// dP = change in position
	Line(dP.h, dP.v);
}


// Combine move and draw

inline void DrawLine(short x1, short y1, short x2, short y2) {
	MoveTo(x1, y1);
	LineTo(x2, y2);
}


inline void DrawLine(const Point& P1, const Point& P2) {
	DrawLine(P1.h, P1.v, P2.h, P2.v);
}


// Get current graphics position
// We assume built in routine to do this: void GetPen(Point* P)
// We define versions with reference parameters

inline void GetPenSpot(int& x, int& y) {
	Point P;
	GetPen(&P);
	x = P.h;
	y = P.v;
}

inline void GetPenSpot(Point& P) {
	GetPen(&P);
}


// Work with pen size

inline void SetPenSize(short xthick, short ythick) {
	PenSize(xthick, ythick);
}


inline void GetPenSize(short& xthick, short& ythick) {
	PenState penstate;
	GetPenState(&penstate);
	xthick = penstate.pnSize.h;
	ythick = penstate.pnSize.v;
}


// Create or modify points

inline void SetPoint(Point& P, short x, short y) {
	P.h = x;
	P.v = y;
}


inline void GetPoint(const Point& P, short& x, short& y) {
	x = P.h;
	y = P.v;
}


inline Point MakePoint(short x, short y) {
	Point P;
	P.h = x;
	P.v = y;
	return P;
}


// Create or modify rectangles

inline void SetRect(Rect& R, short x1, short y1, short x2, short y2) {
	R.left		= x1;
	R.top		= y1;
	R.right		= x2;
	R.bottom	= y2;
}


inline void GetRect(const Rect& R, short& x1, short& y1, short& x2, short& y2) {
	x1 = R.left;
	y1 = R.top;
	x2 = R.right;
	y2 = R.bottom;
}


inline Rect MakeRect(short x1, short y1, short x2, short y2) {
	Rect R;
	R.left		= x1;
	R.top		= y1;
	R.right		= x2;
	R.bottom	= y2;
	return R;	
}

inline Rect MakeRect(Point P1, Point P2) {
	return MakeRect(P1.h, P1.v, P2.h, P2.v);
}

inline void OffsetRect(Rect& R, short dx, short dy) {
	R.left		+= dx;
	R.right		+= dx;
	R.top		+= dy;
	R.bottom	+= dy;
}


inline void InsetRect(Rect& R, short dx, short dy) {
	R.left		+= dx;
	R.right		-= dx;
	R.top		+= dy;
	R.bottom	-= dy;
}


// Test if a Point is in a Rect

inline bool PointInRect(const Point& P, const Rect& R) {
	return PtInRect(P, &R);
}


// Force a, b into increasing order

inline void OrderPair(short& a, short& b) {
	if (a > b) {
		short t = a;
		a = b;
		b = t;
	}
}


// Force rectangle components into increasing order in both x and y

inline void OrderRect(Rect& R) {
	OrderPair(R.left, R.right);
	OrderPair(R.top, R.bottom);
}


// Rectangle drawing routines

inline void FrameRect(Rect R) {
	OrderRect(R);
	FrameRect(&R);
}


inline void FrameRect(short x1, short y1, short x2, short y2) {
	FrameRect(MakeRect(x1, y1, x2, y2));
}


inline void PaintRect(Rect R) {
	OrderRect(R);
	PaintRect(&R);
}


inline void PaintRect(short x1, short y1, short x2, short y2) {
	PaintRect(MakeRect(x1, y1, x2, y2));
}


inline void FillRect(Rect R, PatternPointer pp) {
	OrderRect(R);
	FillRect(&R, pp);
}


inline void FillRect(short x1, short y1, short x2, short y2, PatternPointer pp) {
	FillRect(MakeRect(x1, y1, x2, y2), pp);
}


inline void EraseRect(Rect R) {
	OrderRect(R);
	EraseRect(&R);
}


inline void EraseRect(short x1, short y1, short x2, short y2) {
	EraseRect(MakeRect(x1, y1, x2, y2));
}


inline void InvertRect(Rect R) {
	OrderRect(R);
	InvertRect(&R);
}


inline void InvertRect(short x1, short y1, short x2, short y2) {
	InvertRect(MakeRect(x1, y1, x2, y2));
}


// Oval drawing routines

inline void FrameOval(Rect R) {
	OrderRect(R);
	FrameOval(&R);
}


inline void FrameOval(short x1, short y1, short x2, short y2) {
	FrameOval(MakeRect(x1, y1, x2, y2));
}


inline void PaintOval(Rect R) {
	OrderRect(R);
	PaintOval(&R);
}


inline void PaintOval(short x1, short y1, short x2, short y2) {
	PaintOval(MakeRect(x1, y1, x2, y2));
}


inline void FillOval(Rect R, PatternPointer pp) {
	OrderRect(R);
	FillOval(&R, pp);
}


inline void FillOval(short x1, short y1, short x2, short y2, PatternPointer pp) {
	FillOval(MakeRect(x1, y1, x2, y2), pp);
}


inline void EraseOval(Rect R) {
	OrderRect(R);
	EraseOval(&R);
}


inline void EraseOval(short x1, short y1, short x2, short y2) {
	EraseOval(MakeRect(x1, y1, x2, y2));
}


inline void InvertOval(Rect R) {
	OrderRect(R);
	InvertOval(&R);
}


inline void InvertOval(short x1, short y1, short x2, short y2) {
	InvertOval(MakeRect(x1, y1, x2, y2));
}


// Circle routines

inline Rect CircleRect(Point center, short radius) {
	Rect R;
	
	if (radius < 0)
		radius = -radius;
	
	SetRect(R, center.h - radius, center.v - radius,
			   center.h + radius, center.v + radius);
	
	return R;
}


inline Rect CircleRect(short x, short y, short radius) {
	Rect R;
	
	if (radius < 0)
		radius = -radius;
	
	SetRect(R, x - radius, y - radius,
			   x + radius, y + radius);
	
	return R;
}


inline void FrameCircle(Point center, short radius) {
	FrameOval(&CircleRect(center, radius));
}


inline void FrameCircle(short x, short y, short radius) {
	FrameOval(&CircleRect(x, y, radius));
}


inline void PaintCircle(Point center, short radius) {
	PaintOval(&CircleRect(center, radius));
}


inline void PaintCircle(short x, short y, short radius) {
	PaintOval(&CircleRect(x, y, radius));
}


inline void FillCircle(Point center, short radius, PatternPointer pp) {
	FillOval(&CircleRect(center, radius), pp);
}


inline void FillCircle(short x, short y, short radius, PatternPointer pp) {
	FillOval(&CircleRect(x, y, radius), pp);
}


inline void EraseCircle(Point center, short radius) {
	EraseOval(&CircleRect(center, radius));
}


inline void EraseCircle(short x, short y, short radius) {
	EraseOval(&CircleRect(x, y, radius));
}


inline void InvertCircle(Point center, short radius) {
	InvertOval(CircleRect(center, radius));
}


inline void InvertCircle(short x, short y, short radius) {
	InvertOval(&CircleRect(x, y, radius));
}


// Changing the clipping rectangle

inline void ClipRect(const Rect& R) {
	ClipRect(&R);
}


// Simple color data class
// Macintosh implementation using bytes
// Calls with short are included for backward compatibility


class RGB {
	// The implementation is quite Macintosh dependent
	// Hopefully most of interface can be used on any system
	
	// Macintosh RGBColor stores significant data in high order bytes
	// This is inconvenient from the caller perspective
	// Hence: we use a union to access the bytes easily
	
	union {
		RGBColor rgbcolor;			// Macintosh color struct
		byte     color[6];			// internal view: 6 bytes of data
	};
	
public:
	// default initialization: black
	
	RGB() {
		color[0] = 0;
		color[1] = 0;
		color[2] = 0;
		color[3] = 0;
		color[4] = 0;
		color[5] = 0;
	};
	
	// explicit initialization
	
	RGB(byte r, byte g, byte b) {
		color[0] = r;
		color[1] = 0;
		color[2] = g;
		color[3] = 0;
		color[4] = b;
		color[5] = 0;
	};
	
	
	// set and get data
	
	RGB& Set(byte r, byte g, byte b) {
		color[0] = r;
		color[2] = g;
		color[4] = b;
		
		return *this;
	};
	

	void Get(byte& r, byte& g, byte& b) const {
		r = color[0];
		g = color[2];
		b = color[4];
	};
	

	void Get(short& r, short& g, short& b) const {
		r = color[0];
		g = color[2];
		b = color[4];
	};
	
	
	// member shading routines similar to stand alone versions below
	
	RGB& RedShade		(byte level = 255)	{ return Set(level, 0, 0); }
	RGB& GreenShade		(byte level = 255)	{ return Set(0, level, 0); }
	RGB& BlueShade		(byte level = 255)	{ return Set(0, 0, level); }
	RGB& CyanShade		(byte level = 255)	{ return Set(0, level, level); }
	RGB& MagentaShade	(byte level = 255)	{ return Set(level, 0, level); }
	RGB& YellowShade	(byte level = 255)	{ return Set(level, level, 0); }
	RGB& GrayShade		(byte level = 0)	{ return Set(level, level, level); }
	
	
	// copy constructor
	
	RGB(const RGB& rgb) {
		rgbcolor = rgb.rgbcolor;
	};
	
	
	// operator = for an RGB
	
	RGB& operator= (const RGB& rgb) {
		if (&rgb != this)
			rgbcolor = rgb.rgbcolor;
		
		return *this;
	};
	
	
	// The 4 Macintosh calls below should be replaced
	// when porting to another system
	
	
	// initialization via Macintosh RGBColor
	
	RGB(const RGBColor& thecolor) {
		rgbcolor = thecolor;
	};
	
	
	// operator = for a Macintosh RGBColor
	
	RGB& operator= (const RGBColor& thecolor) {
		rgbcolor = thecolor;
		return *this;
	};
	
	
	// transfer back to a Macintosh RGBColor
	
	operator RGBColor () {					// standard version
		return rgbcolor;
	};
	
	
	operator const RGBColor () const {		// const version
		return rgbcolor;
	};
	
	
	// Request RGB triple from keyboard
	// following the pattern of IOTools
	
	friend RGB RequestRGB(			// with default
		const string&	prompt,			// prompt to user
		const RGB		answer			// default answer
	);
	
	
	friend RGB RequestRGB(			// without default
		const string&	prompt			// prompt to user
	);
	
	
	friend bool ReadingRGB(
		const string&	prompt,			// prompt to user
		RGB&			answer			// user answer if provided
	);
	
	
	// friend functions for conversion of string to RGB
	// following the pattern of IOTools
	
	// at the start position string should contain a triple r g b
	// of integers with each between 0 and 255
	// optionally r g b may be separated by commas as in: r, g, b
	
	friend RGB StringToRGB(
		const string&	source,			// string to read
		int				start,			// start position to read
		int&			next,			// next  position to read
		int&			error			// string IO error code
	);
	
	
	friend RGB StringToRGB(
		const char*		start,			// start position to read
		const char*&	next,			// next  position to read
		int&			error			// string IO error code
	);
	
	
	// member function for conversion of RGB to string
	// wide format: rrr, ggg, bbb
	// otherwise:   r, g, b using minimal space 
	
	string RGBToString(bool wide = false) const;
	
	
	// stream output operator
	
	friend ostream& operator<< (ostream& os, const RGB& rgb) {
		os << rgb.RGBToString(true);
		return os;
	};
	
	
	// Omit: Summer 1997
	//
	// initialization via a string
	// using a name or RGB triplet
	//
	//	RGB(const char* string) {
	//		const char*	next = string;
	//		int			error = 0;
	//		
	//		(*this) = StringToRGB(string, next, error);
	//	};
	//	
	//	// operator = for a string
	//	
	//	RGB& operator= (const char* string) {
	//		const char*	next = string;
	//		int			error = 0;
	//		
	//		return((*this) = StringToRGB(string, next, error));
	//	};
};


// Set and Get ForeGround and BackGround color
// Using one RGB parameter

inline void SetForeColor(const RGB& rgb) {
	RGBForeColor(&(RGBColor(rgb)));
}


inline void SetBackColor(const RGB& rgb) {
	RGBBackColor(&(RGBColor(rgb)));
}


inline void GetForeColor(RGB& rgb) {
	GetForeColor(&(RGBColor(rgb)));
}

inline void GetBackColor(RGB& rgb) {
	GetBackColor(&(RGBColor(rgb)));
}


// Set and Get ForeGround and BackGround color
// Using three byte parameters

inline void SetForeColor(byte r, byte g, byte b) {
	RGB rgb(r, g, b);
	SetForeColor(rgb);
}


inline void SetBackColor(byte r, byte g, byte b) {
	RGB rgb(r, g, b);
	SetBackColor(rgb);
}


inline void GetForeColor(byte& r, byte& g, byte& b) {
	RGB rgb;
	GetForeColor(rgb);
	rgb.Get(r, g, b);
}


inline void GetBackColor(byte& r, byte& g, byte& b) {
	RGB rgb;
	GetBackColor(rgb);
	rgb.Get(r, g, b);
}


inline void GetForeColor(short& r, short& g, short& b) {
	RGB rgb;
	GetForeColor(rgb);
	rgb.Get(r, g, b);
}


inline void GetBackColor(short& r, short& g, short& b) {
	RGB rgb;
	GetBackColor(rgb);
	rgb.Get(r, g, b);
}


// Stand alone routines to generate pure shades:
//		Red, Green, Blue, Cyan, Magenta, Yellow, Gray
// 		Note that GrayShade defaults to level zero to produce black

inline RGB RedShade		(byte level = 255)	{ return RGB(level, 0, 0); }
inline RGB GreenShade	(byte level = 255)	{ return RGB(0, level, 0); }
inline RGB BlueShade	(byte level = 255)	{ return RGB(0, 0, level); }
inline RGB CyanShade	(byte level = 255)	{ return RGB(0, level, level); }
inline RGB MagentaShade	(byte level = 255)	{ return RGB(level, 0, level); }
inline RGB YellowShade	(byte level = 255)	{ return RGB(level, level, 0); }
inline RGB GrayShade	(byte level = 0)	{ return RGB(level, level, level); }


// Pixel Routines

inline void SetColorPixel(short x, short y, const RGB& rgb) {
	SetCPixel(x, y, &(RGBColor(rgb)));
}


inline void GetColorPixel(short x, short y, RGB& rgb) {
	GetCPixel(x, y, &(RGBColor(rgb)));
}


inline void SetColorPixel(short x, short y, byte r, byte g, byte b) {
	RGB rgb(r, g, b);
	SetCPixel(x, y, &(RGBColor(rgb)));
}


inline void GetColorPixel(short x, short y, byte& r, byte& g, byte& b) {
	RGB rgb;
	GetCPixel(x, y, &(RGBColor(rgb)));
	rgb.Get(r, g, b);
}


inline void GetColorPixel(short x, short y, short& r, short& g, short& b) {
	RGB rgb;
	GetCPixel(x, y, &(RGBColor(rgb)));
	rgb.Get(r, g, b);
}

#endif // GRAPHICS_H_
