// Copyright 1997
// College of Computer Science
// Northeastern University Boston MA 02115

// This software may be used for educational purposes as long as this copyright
// notice is retained at the top of all files

// Should this software be modified, the words "Modified from Original" must be
// included as a comment below this notice

// All publication rights are retained.  This software or its documentation may
// not be published in any media either in whole or in part.

///////////////////////////////////////////////////////////////////////////////

// Polar2D.cp
// minimal classes for Polar2D objects in radians or degrees

///////////////////////////////////////////////////////////////////////////////

#include "Polar2D.h"

// PolarRadians operations

// auto conversion to Point2D
PolarRadians::operator Point2D() const {
	double x = radius * cos(radians);
	double y = radius * sin(radians);
	return Point2D(x, y);
}

// conversion: PolarDegrees to PolarRadians
void PolarRadians::Set(const PolarDegrees& P) {
	radius  = P.radius;
	radians = degrees_to_radians * P.degrees;
}

// conversion: Point2D to PolarRadians
void PolarRadians::Set(const Point2D& P) {
	double x;
	double y;
	P.Get(x, y);
	
	if (x == 0 && y == 0) {
		radius  = 0;
		radians = 0;
	}
	else {
		radius  = sqrt(x*x + y*y);
		radians = atan2(y, x);
	}	
}

// conversion: PolarRadians to PolarDegrees
void PolarRadians::Get(PolarDegrees& P) const {
	P.radius  = radius;
	P.degrees = radians_to_degrees * radians;
}


// PolarDegrees operations

// auto conversion to Point2D
PolarDegrees::operator Point2D() const {
	double radians = degrees_to_radians * degrees;
	double x = radius * cos(radians);
	double y = radius * sin(radians);
	return Point2D(x, y);
}

// conversion: PolarRadians to PolarDegrees
void PolarDegrees::Set(const PolarRadians& P) {
	radius  = P.radius;
	degrees = radians_to_degrees * P.radians;
}

// conversion: Point2D to PolarDegrees
void PolarDegrees::Set(const Point2D& P) {
	double x;
	double y;
	P.Get(x, y);
	
	if (x == 0 && y == 0) {
		radius  = 0;
		degrees = 0;
	}
	else {
		radius  = sqrt(x*x + y*y);
		degrees = atan2deg(y, x);
	}	
}
// conversion: PolarDegrees to PolarRadians
void PolarDegrees::Get(PolarRadians& P) const {
	P.radius  = radius;
	P.radians = degrees_to_radians * degrees;
}
