// Copyright 1997
// College of Computer Science
// Northeastern University Boston MA 02115

// This software may be used for educational purposes as long as this copyright
// notice is retained at the top of all files

// Should this software be modified, the words "Modified from Original" must be
// included as a comment below this notice

// All publication rights are retained.  This software or its documentation may
// not be published in any media either in whole or in part.

///////////////////////////////////////////////////////////////////////////////

// Polar2D.h
// minimal classes for Polar2D objects in radians or degrees

///////////////////////////////////////////////////////////////////////////////

#include "Basic2D.h"

// forward declarations

class PolarRadians;
class PolarDegrees;


// declarations

class PolarRadians {
	double radius;		// distance from origin
	double radians;		// angle in radians
	
public:
	friend class PolarDegrees;
	
	// auto conversion to Point2D
	operator Point2D() const;
	
	
	// set and get functions
	
	// direct set
	void Set(double Radius, double Radians) {
		radius  = Radius;
		radians = Radians;
	};
	
	// conversion: PolarDegrees to PolarRadians
	void Set(const PolarDegrees& P);
	
	// conversion: Point2D to PolarRadians
	void Set(const Point2D& P);
	
	// direct get
	void Get(double& Radius, double& Radians) const {
		Radius  = radius;
		Radians = radians;
	};
	
	// conversion: PolarRadians to PolarDegrees
	void Get(PolarDegrees& P) const;
	
	// conversion: PolarRadians to Point2D
	void Get(Point2D& P) const {
		// finesse: apply operator Point2D
		P = *this;
	};
	
	
	// constructors
	
	PolarRadians() : radius(0), radians(0) { };
	
	PolarRadians(double Radius, double Radians)
		: radius(Radius), radians(Radians) { };
	
	PolarRadians(const PolarDegrees& P) { Set(P); };
	
	PolarRadians(const Point2D& P) { Set(P); };
};


class PolarDegrees {
	double radius;		// distance from origin
	double degrees;		// angle in degrees
	
public:
	friend class PolarRadians;
	
	// auto conversion to Point2D
	operator Point2D() const;
	
	
	// set and get functions
	
	// direct set
	void Set(double Radius, double Degrees) {
		radius  = Radius;
		degrees = Degrees;
	};
	
	// conversion: PolarRadians to PolarDegrees
	void Set(const PolarRadians& P);
	
	// conversion: Point2D to PolarDegrees
	void Set(const Point2D& P);
	
	// direct get
	void Get(double& Radius, double& Degrees) const {
		Radius  = radius;
		Degrees = degrees;
	};
	
	// conversion: PolarDegrees to PolarRadians
	void Get(PolarRadians& P) const;
	
	// conversion: PolarDegrees to Point2D
	void Get(Point2D& P) const {
		// finesse: apply operator Point2D
		P = *this;
	};
	
	
	// constructors
	
	PolarDegrees() : radius(0), degrees(0) { };
	
	PolarDegrees(double Radius, double Degrees)
		: radius(Radius), degrees(Degrees) { };
	
	PolarDegrees(const PolarRadians& P) { Set(P); };
	
	PolarDegrees(const Point2D& P) { Set(P); };
};
