// Copyright 1997
// College of Computer Science
// Northeastern University Boston MA 02115

// This software may be used for educational purposes as long as this copyright
// notice is retained at the top of all files

// Should this software be modified, the words "Modified from Original" must be
// included as a comment below this notice

// All publication rights are retained.  This software or its documentation may
// not be published in any media either in whole or in part.

///////////////////////////////////////////////////////////////////////////////

//	Scale.cp

///////////////////////////////////////////////////////////////////////////////

#include "Scale.h"

// LinearScale functions

// set: map interval a, b to ia, ib
//
// transformation is:
//		t --> ia + ((t - a) / (b - a)) * (ib - ia)
// or
//		t --> ((ib - ia) / (b - a)) * t + (ia - ((ib - ia) / (b - a)) * a)

void LinearScale::Set(double a, double b, short ia, short ib) {
	// if interval endpoints coincide 
	// return best possible transform
	// map (a + b) / 2 to (ia + ib)/2
	// with factor set to 1
	if ((a == b) || (ia == ib)) {
		factor = 1;
		offset = (double(ia + ib) - (a + b)) / 2;
		return;
	}
	
	// otherwise compute correct transform in the general case
	factor = double(ib - ia) / (b - a);
	offset = double(ia) - factor * a;
}


// LogScale functions

// map interval a, b to ia, ib

void LogScale::Set(double a, double b, short ia, short ib) {
	// error if a <= 0 or b <= 0 since logarithm is invalid
	// abort in this case via assert macro
	assert (a > 0 && b > 0);
	
	// if ok then set linear scale S usings logs of a and b
	S.Set(log(a), log(b), ia, ib);
}
