// Copyright 1997
// College of Computer Science
// Northeastern University Boston MA 02115

// This software may be used for educational purposes as long as this copyright
// notice is retained at the top of all files

// Should this software be modified, the words "Modified from Original" must be
// included as a comment below this notice

// All publication rights are retained.  This software or its documentation may
// not be published in any media either in whole or in part.

///////////////////////////////////////////////////////////////////////////////

//	Scale2D.h

///////////////////////////////////////////////////////////////////////////////

#ifndef SCALE2D_H_
#define SCALE2D_H_

#include "Scale.h"
#include "Basic2D.h"
#include "Graphics.h"

// abstract base class for 2D scale operation

class Scale2D {
	// no data in this abstract base class
	
public:
	// a 2D scale class must have an operator() that takes a Point2D to a Point
	virtual Point operator() (const Point2D&) const = 0;

	// a scale class must have an inverse function to its operator()
	virtual Point2D Inverse(const Point&) const = 0;
};


// class to define a linear 2D scale function

class LinearScale2D : public Scale2D {
	LinearScale sx;
	LinearScale sy;
	
public:
	// set functions

	// supply factors and offsets
	void Set(double FactorX, double OffsetX, double FactorY, double OffsetY) {
		sx.Set(FactorX, OffsetX);
		sy.Set(FactorY, OffsetY);
	};
	
	// map Rect2D Bounds to Rect Limits
	void Set(const Rect2D& Bounds, const Rect& Limits);
	
	// map Rect2D Bounds to Rect Limits
	// require that the x-y scale factors have the same absolute value
	// so that the true geometric shape of objects is maintained
	void SetTrueShape(const Rect2D& Bounds, const Rect& Limits);
	
	
	// constructors
	
	// default
	LinearScale2D() : sx(), sy() { };
	
	// supply factors and offsets
	LinearScale2D(double FactorX, double OffsetX, double FactorY, double OffsetY)
		: sx(FactorX, OffsetX), sy(FactorY, OffsetY) { };
	
	// map Rect2D Bounds to Rect Limits
	LinearScale2D(const Rect2D& Bounds, const Rect& Limits)
		{ Set(Bounds, Limits); };
	
	
	// scaling transform operator
	Point operator() (const Point2D& P) const {
		double X;
		double Y;
		
		P.Get(X, Y);
		
		return MakePoint(sx(X), sy(Y));
	};
	
	// scaling inverse transform operator
	Point2D Inverse(const Point& P) const {
		short X;
		short Y;
		
		GetPoint(P, X, Y);
		
		return Point2D( sx.Inverse(X), sy.Inverse(Y) );
	};
};
	
#endif // SCALE2D_H_