// Copyright 1997
// College of Computer Science
// Northeastern University Boston MA 02115

// This software may be used for educational purposes as long as this copyright
// notice is retained at the top of all files

// Should this software be modified, the words "Modified from Original" must be
// included as a comment below this notice

// All publication rights are retained.  This software or its documentation may
// not be published in any media either in whole or in part.

///////////////////////////////////////////////////////////////////////////////

// Text.cp

#include "Text.h"
#include "Strings.h"		// Macintosh C string to Pascal string converter


FontData& FontData::Select() {
	// copy font name to C string
	int amount = name.length();				// find length of name
	
	if (amount == 0)						// exit early: no name
		return *this;
	
	char* ptr = new char[amount + 1];		// allocate
	
	strcpy(ptr, name.c_str());				// copy
	
	// convert to Pascal string format using same space
	StringPtr s = c2pstr(ptr);
	
	// find the font id from the Pascal formatted string
	GetFNum(s, &fontid);
	
	// deallocate string space which is no longer needed
	delete [] ptr;
	
	// Select font with given size and style for the drawing window
	
	TextFont(fontid);
	TextSize(size);
	TextFace(style);
	
	// Now obtain the rest of the font parameters from the runtime system
	
	FontInfo fontinfo;
	GetFontInfo(&fontinfo);
	
	ascent	= fontinfo.ascent;
	descent	= fontinfo.descent;
	leading	= fontinfo.leading;
	widmax	= fontinfo.widMax;
	
	height	= ascent + descent + leading;
	
	return *this;
}


void ShowString (const string& data, short justify) {
	// copy data to C string
	int amount = data.length();				// find length of data
	
	if (amount == 0)						// exit early: no data
		return;
	
	char* ptr = new char[amount + 1];		// allocate
	
	strcpy(ptr, data.c_str());				// copy
	
	// convert to Pascal string format using same space
	StringPtr s = c2pstr(ptr);
	
	short width;
	
	// switch for justification
	switch (justify) {
		case Right:
			width = StringWidth(s);
			Move(-width, 0);
			break;
		
		case Center:
			width = StringWidth(s) / 2;
			Move(-width, 0);
			break;
		
		default:
			break;
	}	
	
	DrawString(s);
	
	// deallocate string space which is no longer needed
	delete [] ptr;
}
