// Copyright 1997
// College of Computer Science
// Northeastern University Boston MA 02115

// This software may be used for educational purposes as long as this copyright
// notice is retained at the top of all files

// Should this software be modified, the words "Modified from Original" must be
// included as a comment below this notice

// All publication rights are retained.  This software or its documentation may
// not be published in any media either in whole or in part.

///////////////////////////////////////////////////////////////////////////////

// Text.h

///////////////////////////////////////////////////////////////////////////////

// Changes: Summer 1997
// Use C++ string in place of CharBuffer in implementation

///////////////////////////////////////////////////////////////////////////////

#ifndef TEXT_H_
#define TEXT_H_

// #include "ChBuffer.h"	Omit!

#include <string>
#include "Enum.h"


class FontData {
	string		name;		// name as string
	short		size;		// type size
	short		style;		// type style
	
	// The following fields are valid after the font has been selected
	// Before that the fields are zero
	short		fontid;		// font id number
	short		ascent;		// ascent  above baseline
	short		descent;	// descent below baseline
	short		leading;	// leading between lines
	short		height;		// ascent + descent + leading
	short		widmax;		// maximum character width
	
	// Reset to newly initialized state
	
	void Reset() {
		if (size < 1)
			size = 1;
		
		if (style < 0)
			style = 0;
		
		fontid		= 0;
		ascent 		= 0;
		descent		= 0;
		leading		= 0;
		height		= 0;
		widmax		= 0;
	}
	
public:
	FontData(const string& Name, short Size = 12, short Style = 0)
		: name(Name), size(Size), style(Style) { Reset(); };
	
	
	// Set ... functions with parameters ... return font object
	
	FontData& Set(const string& Name, short Size = 12, short Style = 0) {
		name	= Name;
		size	= Size;
		style	= Style;
		Reset();
		
		return *this;
	}
	
	FontData& Name(const string& theName) {
		name	= theName;
		Reset();
		
		return *this;
	};
	
	FontData& Size(short theSize) {
		size	= theSize;
		Reset();
		
		return *this;
	}
	
	FontData& Style(short theStyle) {
		style	= theStyle;
		Reset();
		
		return *this;
	}
	
	
	// Get ... functions without parameters and with data return
	
	string Name(void) const {
		return name;
	};
	
	short Size(void) const {
		return size;
	};
	
	short Style(void) const {
		return style;
	};
	
	short FontID(void) const {
		return fontid;
	};
	
	short Ascent(void) const {
		return ascent;
	};
	
	short Descent(void) const {
		return descent;
	};
	
	short Leading(void) const {
		return leading;
	};
	
	short Height(void) const {
		return height;
	};
	
	short MaxWidth(void) const {
		return widmax;
	};
	
	// Select font for graphics window and return font object
	FontData& Select();
};


// Draw string in graphics window

void ShowString (const string& data, short justify = Left);


// Mode for erasing text by writing over with bit clear on
inline void WhiteText(void) {
	TextMode(srcBic);
}

// Normal text writing mode ... resets after WhiteText

inline void BlackText(void) {
	TextMode(srcOr);
}

#endif // TEXT_H_
